library nueli.core.config;

import 'dart:math';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:nueli/nueli.dart';

import '../bucket/--bucket.dart';
import '../data.dart';

part 'decor.dart';
part 'ink.dart';
part 'photo.dart';
part 'radii.dart';
part 'shadow.dart';
part 'tranx.dart';
part 'txt.dart';
part 'x3D.dart';

abstract class _Config<T extends _Config<T>> with Add<T>, Subtract<T>, Multiply<T, double>, OR<T, dynamic>, AND<T, dynamic> {}

abstract class Config<T extends Config<T, Data>, Data extends ConfigData<Data>> extends _Config<T> {
  final Data fallback;
  Data _data;

  Data get data => _data;
  set data(Data? data) => _data = data ?? fallback;

  Config(this.fallback) : _data = fallback;

  T withChild(Widget? child) => this & child;

  @mustCallSuper
  T lerp(data) {
    if (data is T) this.data = this.data & data.data;
    return this as T..data = this.data & data;
  }

  @mustCallSuper
  T operator &(data) => lerp(data);

  @mustCallSuper
  T operator |(data) => data is T ? data.lerp(this) : this as T;

  Widget call();
}

class ConfigData<T extends ConfigData<T>> with OR<T, dynamic>, AND<T, dynamic> {
  Widget? child;

  ConfigData(this.child);

  T lerp(data) {
    if (data is T) child = data.child;
    if (data is Widget) child = data;
    return this as T;
  }

  @mustCallSuper
  T operator *(double operand) => this as T;

  @mustCallSuper
  T operator &(data) => lerp(data);

  @mustCallSuper
  T operator |(data) => data is T ? data.lerp(this) : this as T;
}

class Attr {
  final Symbol embedded;
  dynamic value;

  Attr(this.embedded);

  Attr operator [](dynamic value) {
    this.value = value;
    return this;
  }
}
