part of nueli.core.config;

RadiiConfig get $radii => RadiiConfig();

class RadiiConfig extends Config<RadiiConfig, Radii> {
  double? _tlX;
  double? _tlY;
  double? _trX;
  double? _trY;
  double? _brX;
  double? _brY;
  double? _blX;
  double? _blY;

  RadiiConfig() : super(Radii());

  RadiiConfig _config({
    double? tlX,
    double? tlY,
    double? trX,
    double? trY,
    double? brX,
    double? brY,
    double? blX,
    double? blY,
  }) {
    _tlX = tlX ?? _tlX;
    _tlY = tlY ?? _tlY;
    _trX = trX ?? _trX;
    _trY = trY ?? _trY;
    _brX = brX ?? _brX;
    _brY = brY ?? _brY;
    _blX = blX ?? _blX;
    _blY = blY ?? _blY;
    return this;
  }

  // input

  RadiiConfig tl_x(double value) => _config(tlX: value); // ignore: non_constant_identifier_names
  RadiiConfig tl_y(double value) => _config(tlY: value); // ignore: non_constant_identifier_names
  RadiiConfig tr_x(double value) => _config(trX: value); // ignore: non_constant_identifier_names
  RadiiConfig tr_y(double value) => _config(trY: value); // ignore: non_constant_identifier_names
  RadiiConfig br_x(double value) => _config(brX: value); // ignore: non_constant_identifier_names
  RadiiConfig br_y(double value) => _config(brY: value); // ignore: non_constant_identifier_names
  RadiiConfig bl_x(double value) => _config(blX: value); // ignore: non_constant_identifier_names
  RadiiConfig bl_y(double value) => _config(blY: value); // ignore: non_constant_identifier_names

  RadiiConfig top_left(double x, [double? y]) => tl_x(x).tl_y(y ?? x); // ignore: non_constant_identifier_names
  RadiiConfig top_right(double x, [double? y]) => tr_x(x).tr_y(y ?? x); // ignore: non_constant_identifier_names
  RadiiConfig bottom_right(double x, [double? y]) => br_x(x).br_y(y ?? x); // ignore: non_constant_identifier_names
  RadiiConfig bottom_left(double x, [double? y]) => bl_x(x).bl_y(y ?? x); // ignore: non_constant_identifier_names

  RadiiConfig left(double value) => tl_y(value).bl_y(value);
  RadiiConfig right(double value) => tr_y(value).br_y(value);
  RadiiConfig top(double value) => tl_x(value).tr_x(value);
  RadiiConfig bottom(double value) => bl_x(value).br_x(value);

  RadiiConfig vert(double top, double bottom) => top_left(top).top_right(top).bottom_left(bottom).bottom_right(bottom);
  RadiiConfig horz(double left, double right) => top_left(left).top_right(right).bottom_left(left).bottom_right(right);

  RadiiConfig corners(double tl, double tr, double br, double bl) => top_left(tl).top_right(tr).bottom_right(br).bottom_left(bl); // ignore: non_constant_identifier_names
  RadiiConfig sides(double left, double right, double top, double bottom) => this.left(left).right(right).top(top).bottom(bottom); // ignore: non_constant_identifier_names

  RadiiConfig all(double value) => left(value).right(value).top(value).bottom(value);
  RadiiConfig get all_8 => all(8);
  RadiiConfig get all_16 => all(16);
  RadiiConfig get all_24 => all(24);
  RadiiConfig get all_52 => all(52);
  RadiiConfig get all_108 => all(108);
  RadiiConfig get zero => all(0);

  // operation
  @override
  RadiiConfig operator *(double factor) {
    final radiiAp = _RadiiApprox(this);
    return _config(
      tlX: radiiAp.tlX * factor,
      tlY: radiiAp.tlY * factor,
      trX: radiiAp.trX * factor,
      trY: radiiAp.trY * factor,
      brX: radiiAp.brX * factor,
      brY: radiiAp.brY * factor,
      blX: radiiAp.blX * factor,
      blY: radiiAp.blY * factor,
    );
  }

  @override
  RadiiConfig operator -(RadiiConfig operand) {
    final radiiAp = _RadiiApprox(this);
    final radiiAp2 = _RadiiApprox(operand);
    return _config(
      tlX: radiiAp.tlX - radiiAp2.tlX,
      tlY: radiiAp.tlY - radiiAp2.tlY,
      trX: radiiAp.trX - radiiAp2.trX,
      trY: radiiAp.trY - radiiAp2.trY,
      brX: radiiAp.brX - radiiAp2.brX,
      brY: radiiAp.brY - radiiAp2.brY,
      blX: radiiAp.blX - radiiAp2.blX,
      blY: radiiAp.blY - radiiAp2.blY,
    );
  }

  @override
  RadiiConfig operator +(RadiiConfig operand) {
    final radiiAp = _RadiiApprox(this);
    final radiiAp2 = _RadiiApprox(operand);
    return _config(
      tlX: radiiAp.tlX + radiiAp2.tlX,
      tlY: radiiAp.tlY + radiiAp2.tlY,
      trX: radiiAp.trX + radiiAp2.trX,
      trY: radiiAp.trY + radiiAp2.trY,
      brX: radiiAp.brX + radiiAp2.brX,
      brY: radiiAp.brY + radiiAp2.brY,
      blX: radiiAp.blX + radiiAp2.blX,
      blY: radiiAp.blY + radiiAp2.blY,
    );
  }

  @override
  RadiiConfig lerp(radii) {
    if (radii is RadiiConfig)
      _config(
        tlX: radii._tlX ?? _tlX,
        tlY: radii._tlY ?? _tlY,
        trX: radii._trX ?? _trX,
        trY: radii._trY ?? _trY,
        blX: radii._blX ?? _blX,
        blY: radii._blY ?? _blY,
        brX: radii._brX ?? _brX,
        brY: radii._brY ?? _brY,
      );
    return super.lerp(radii);
  }

  // output
  BorderRadius get toBorderRadius {
    final radiiAp = _RadiiApprox(this);
    return BorderRadius.only(
      topLeft: Radius.elliptical(radiiAp.tlX, radiiAp.tlY),
      topRight: Radius.elliptical(radiiAp.trX, radiiAp.trY),
      bottomLeft: Radius.elliptical(radiiAp.blX, radiiAp.blY),
      bottomRight: Radius.elliptical(radiiAp.brX, radiiAp.brY),
    );
  }

  Widget call() {
    return Container(
      child: data.child,
      decoration: BoxDecoration(borderRadius: toBorderRadius),
      clipBehavior: Clip.antiAliasWithSaveLayer,
    );
  }
}

class Radii extends ConfigData<Radii> {
  Radii({Widget? child}) : super(child);
}

class RadiiTween extends Tween<RadiiConfig> {
  RadiiTween({RadiiConfig? begin, RadiiConfig? end}) : super(begin: begin, end: end);

  @override
  RadiiConfig transform(double value) => (begin ?? $radii) + ((end ?? $radii) - (begin ?? $radii)) * value;
}

class RadiiTransition extends AnimatedWidget {
  final Widget? child;
  RadiiTransition({required Animation<RadiiConfig> radii, this.child}) : super(listenable: radii);
  Animation<RadiiConfig> get radii => listenable as Animation<RadiiConfig>;

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: listenable,
      builder: (c, child) => radii.value.withChild(child)(),
    );
  }
}

class _RadiiApprox {
  final RadiiConfig radii;
  _RadiiApprox(this.radii);

  double get tlX => radii._tlX ?? 0;
  double get tlY => radii._tlX ?? 0;
  double get trX => radii._trX ?? 0;
  double get trY => radii._trX ?? 0;
  double get blX => radii._blX ?? 0;
  double get blY => radii._blX ?? 0;
  double get brX => radii._brX ?? 0;
  double get brY => radii._brX ?? 0;
}
