part of nueli.core.config;

AceConfig get $ace => AceConfig();

class AceConfig extends Config<AceConfig, Ace> {
  List<AceConfig> _children = const [];
  Color? _color;
  Offset? _offset;
  double? _spread;
  double? _blur;

  AceConfig() : super(Ace());

  AceConfig _config({
    Color? color,
    Offset? offset,
    double? spread,
    double? blur,
  }) {
    _color = color ?? _color;
    _offset = offset ?? _offset;
    _spread = spread ?? _spread;
    _blur = blur ?? _blur;
    return this;
  }

  AceConfig color(Color color) => _config(color: color);
  AceConfig grow(double x, [double? y]) => _config(offset: Offset(x, y ?? x));
  AceConfig blur(double blur) => _config(blur: blur); // ignore: non_constant_identifier_names
  AceConfig spread(double spread) => _config(spread: spread); // ignore: non_constant_identifier_names
  AceConfig glow(double glow) => _config(offset: Offset.zero, blur: glow);

  AceConfig elevate(double elevation) => _config(blur: elevation, spread: -0.5, offset: Offset(elevation / 8, elevation / 8));
  AceConfig raise(double float) => _config(blur: float / 6, spread: -0.5, offset: Offset(float, float));

  AceConfig withPosition(DecorationPosition? position) => this & position;

  @override
  AceConfig operator *(double factor) {
    final aceAp = _AceApprox(this);
    _children.map((e) => e * factor).toList();
    return _config(
      color: aceAp.color * factor,
      offset: aceAp.offset * factor,
      spread: aceAp.spread * factor,
      blur: aceAp.blur * factor,
    );
  }

  @override
  AceConfig operator -(AceConfig operand) {
    final aceAp = _AceApprox(this);
    final aceAp2 = _AceApprox(operand);
    _children.map((e) => e - operand).toList();
    return _config(
      color: aceAp.color - aceAp2.color,
      offset: aceAp.offset - aceAp2.offset,
      spread: aceAp.spread - aceAp2.spread,
      blur: aceAp.blur - aceAp2.blur,
    );
  }

  @override
  AceConfig operator +(AceConfig operand) {
    final aceAp = _AceApprox(this);
    final aceAp2 = _AceApprox(operand);
    _children.map((e) => e + operand).toList();
    return _config(
      color: aceAp.color + aceAp2.color,
      offset: aceAp.offset + aceAp2.offset,
      spread: aceAp.spread + aceAp2.spread,
      blur: aceAp.blur + aceAp2.blur,
    );
  }

  @override
  AceConfig lerp(shadow) {
    if (shadow is AceConfig)
      _config(
        color: shadow._color ?? _color,
        blur: shadow._blur ?? _blur,
        spread: shadow._spread ?? _spread,
        offset: shadow._offset ?? _offset,
      );
    return super.lerp(shadow);
  }

  BoxShadow get toBoxShadow {
    final aceAp = _AceApprox(this);
    return BoxShadow(
      color: aceAp.color,
      offset: aceAp.offset,
      blurRadius: aceAp.blur,
      spreadRadius: aceAp.spread,
    );
  }

  List<BoxShadow> get toBoxShadowDeep => _children.map((e) => e.toBoxShadow).toList()..add(this.toBoxShadow);

  AceConfig save() => AceConfig().._children.addAll([...this._children, this.._children = []]);

  Widget call() {
    return DecoratedBox(
      decoration: BoxDecoration(boxShadow: toBoxShadowDeep),
      child: data.child,
      position: data.position ?? DecorationPosition.background,
    );
  }
}

class Ace extends ConfigData<Ace> {
  DecorationPosition? position;

  Ace({Widget? child, this.position}) : super(child);

  Ace lerp(data) {
    if (data is DecorationPosition) position = data;

    if (data is Ace) position = data.position;
    return super.lerp(data);
  }
}

class AceTween extends Tween<AceConfig> {
  AceTween({AceConfig? begin, AceConfig? end}) : super(begin: begin, end: end);

  @override
  AceConfig transform(double value) => (begin ?? $ace) + ((end ?? $ace) - (begin ?? $ace)) * value;
}

class AceTransition extends AnimatedWidget {
  final Widget? child;
  AceTransition({required Animation<AceConfig> ace, this.child}) : super(listenable: ace);
  Animation<AceConfig> get ace => listenable as Animation<AceConfig>;

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: listenable,
      builder: (c, child) => ace.value.withChild(child)(),
    );
  }
}

class _AceApprox {
  final AceConfig shadow;

  _AceApprox(this.shadow);
  Color get color => shadow._color ?? palette.none;
  Offset get offset => shadow._offset ?? Offset.zero;
  double get spread => shadow._spread ?? 0;
  double get blur => shadow._blur ?? 0;
}
