library nueli.ui;

import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:need_resume/need_resume.dart';

import '../core/--core.dart';

part 'screen.dart';
part 'struct.dart';

const ui = const _UITools();

mixin UI on StatefulWidget {
  @mustCallSuper
  Size size(double width, [double? height]) => ui.size(width, height);

  @mustCallSuper
  BuildContext get context => ui.globalContext ?? context;

  @mustCallSuper
  Widget get empty => ui.empty;

  @mustCallSuper
  Widget gap(double size) => ui.gap(size);

  @mustCallSuper
  Widget dot(double size, [Color? color]) => ui.dot(size, color ?? palette.text);

  @mustCallSuper
  Widget lineH(double thickness, [Color? color]) => ui.lineH(thickness, color ?? palette.text);

  @mustCallSuper
  Widget lineV(double thickness, [Color? color]) => ui.lineV(thickness, color ?? palette.text);

  @mustCallSuper
  Expanded flex(int flex) => ui.flex(flex);

  @mustCallSuper
  setState(Function() fn) => (ui.globalSetState ?? fn())(fn);
}

mixin UIState<T extends UI> on State<T> {
  @mustCallSuper
  Size size(double width, [double? height]) => ui.size(width, height);

  @mustCallSuper
  Widget get empty => ui.empty;

  @mustCallSuper
  Widget gap(double size) => _UITools().gap(size);

  @mustCallSuper
  Widget dot(double size, [Color? color]) => _UITools().dot(size, color ?? palette.text);

  @mustCallSuper
  Widget lineH(double thickness, [Color? color]) => _UITools().lineH(thickness, color ?? palette.text);

  @mustCallSuper
  Widget lineV(double thickness, [Color? color]) => _UITools().lineV(thickness, color ?? palette.text);

  @mustCallSuper
  Expanded flex(int flex) => _UITools().flex(flex);

  @mustCallSuper
  initTools() {
    ui.globalContext = context;
    ui.globalSetState = setState;
  }

  @override
  void initState() {
    initTools();
    super.initState();
  }

  @mustCallSuper
  Widget build(BuildContext context) {
    initTools();
    return empty;
  }
}

class _UITools {
  static BuildContext? _globalContext;
  static Function(Function())? _globalSetState;
  static const Widget _empty = SizedBox();

  const _UITools();
  set globalContext(BuildContext? context) => _UITools._globalContext = context;
  set globalSetState(Function(Function())? setState) => _UITools._globalSetState = setState;

  Size size(double width, [double? height]) => Size(width, height ?? width);

  BuildContext? get globalContext => _globalContext;

  Function(Function())? get globalSetState => _globalSetState;

  Widget get empty => _empty;

  Widget gap(double size) => SizedBox(width: size, height: size);

  Widget lineH(double thickness, [Color? color]) => Container(
        height: thickness,
        width: double.infinity,
        decoration: $decor.paint(color ?? palette.accent).radii($radii.all(thickness)).toDecoration,
      );

  Widget lineV(double thickness, [Color? color]) => Container(
        width: thickness,
        height: double.infinity,
        decoration: $decor.paint(color ?? palette.accent).radii($radii.all(thickness)).toDecoration,
      );

  Widget dot(double size, [Color? color]) => Container(
        width: size,
        height: size,
        decoration: $decor.paint(color ?? palette.accent).radii($radii.all(size)).toDecoration,
      );

  Expanded flex(int flex) => Expanded(child: empty, flex: flex);
}

extension ExpandedUI on Expanded {
  Expanded operator &(Widget child) => Expanded(child: child, key: key, flex: flex);
}
