part of nueli.ui;

/// <h1>>> Struct </h1>
@immutable
abstract class Struct extends StatefulWidget with UI {
  Struct({Key? key}) : super(key: key);

  Widget get body;

  @mustCallSuper
  Widget populate() => body;

  StructState<Struct> createState() => StructState();
}

/// Struct state
@protected
class StructState<T extends Struct> extends State<T> with UIState<T> {
  @mustCallSuper
  Widget build(BuildContext context) {
    super.build(context);
    return widget.populate();
  }
}

///<h1>>> Anim </h1>
mixin Anim on Struct {
  final Ref<AnimationController> _controller = $();

  Duration get duration => Duration(milliseconds: 500);
  Duration get reverseDuration => duration;
  Curve get curve => Curves.easeOutExpo;
  Curve get reverseCurve => curve;

  onComplete() {}

  /// Do not override
  Animation<double> get animation => CurvedAnimation(parent: _controller.data!, curve: curve, reverseCurve: reverseCurve);

  /// Do not override
  forward({double? from}) => _controller.data!.forward(from: from);

  /// Do not override
  reverse({double? from}) => _controller.data!.reverse(from: from);

  /// Do not override
  animateTo(double target, {Duration? duration, Curve curve = Curves.linear}) => _controller.data!.animateTo(target);

  /// Do not override
  repeat({double? min, double? max, bool reverse = false, Duration? period}) {
    return _controller.data!.repeat(min: min, max: max, reverse: reverse, period: period ?? Duration(milliseconds: 500));
  }

  AnimState<Anim> createState() => AnimState();
}

class AnimState<T extends Anim> extends StructState<T> with TickerProviderStateMixin {
  AnimationStatusListener get _status {
    return (s) => s == AnimationStatus.completed || s == AnimationStatus.dismissed ? widget.onComplete() : {};
  }

  initTools() {
    widget._controller.data = widget._controller.data ??
        AnimationController(
          vsync: this,
          duration: widget.duration,
          reverseDuration: widget.reverseDuration,
        )
      ..addStatusListener(_status);
    super.initTools();
  }

  dispose() {
    widget._controller.data!.removeStatusListener(_status);
    widget._controller.data!.dispose();
    super.dispose();
  }
}
